#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include "calculations.h"
#include "file_io.h"

void display_help(){
  fprintf(stderr, "Format for command line options is \'aasc [options] molname diel_external diel_internal\'\n");
  fprintf(stderr, "Note that optional arguments must be specified in 'option=arg' format\n");
  fprintf(stderr, "molname is the name of the molecule\n");
  fprintf(stderr, "diel_external is the exterior dielectric charge\n");
  fprintf(stderr, "diel_internal is the interior dielectric charge\n");
  fprintf(stderr, "[options] can be any combination of the following:\n\n");
  fprintf(stderr, "\t--alpha                 \t\t<value>\n");
  fprintf(stderr, "\t--projection_length     \t\t<length>\n");
  fprintf(stderr, "\t--triangulation_density \t\t<triangles>\n");
  fprintf(stderr, "\t--nanoshaper_config_file\t\t<configuration file>\n");
  fprintf(stderr, "\t--water_probe_radius    \t\t<radius>\n");
  fprintf(stderr, "\t--residue_list          \t\t<residue list file>\n");
  fprintf(stderr, "\t--write_ef              \t\tWrites electric field -- optional filename\n");
  fprintf(stderr, "\t--write_asc             \t\tWrites apparent surface charge -- optional filename\n");
  fprintf(stderr, "\t--total_surface_charge  \t\tSanity Check -- Computes total surface charge from triangulation\n");
  fprintf(stderr, "\t--solvation_energy      \t\tCalculates solvation free energy\n\n");
  fprintf(stderr, "\tIf no additional parameter flag is given, --solvation_energy is assumed\n");;
}

static int fix_electrostatic_radius_flag;
static int write_electric_field_flag;
static int write_apparent_surface_charge_flag;
static int solvation_energy_flag;
static int total_surface_charge_flag;
static int residue_list_flag;
static int msms_flag;

int main (int argc, char **argv)
{
  char *electric_field_write_file = calloc(256, sizeof(char));
  char *apparent_surface_charge_write_file = calloc(256, sizeof(char));
  char *residue_list_file = calloc(256, sizeof(char));
  char *nano_conf_file = '\0';

  double alpha = 0.580127;
  double triangulation_density = 2;
  double water_probe_radius = 1.4;
  double projection_length = 0;
  double A = 0;

  static struct option long_options[] = {
    {"alpha", required_argument, NULL, 'a'},
    {"projection_length", required_argument, NULL, 'b'},
    {"triangulation_density", required_argument, NULL, 'c'},
    {"water_probe_radius", required_argument, NULL, 'd'},
    {"residue_list", required_argument, NULL, 'j'},
    {"nanoshaper_config_file", required_argument, NULL, 'f'},
    {"write_ef", optional_argument, NULL, 'g'},
    {"write_asc", optional_argument, NULL, 'i'},
    {"help", no_argument, NULL, 'h'},
    {"solvation_energy", no_argument, &solvation_energy_flag, 'k'},
    {"total_surface_charge", no_argument, &total_surface_charge_flag, 'l'},
    {"msms", no_argument, &msms_flag, 1},
    {0, 0, 0, 0}
  };

  int opt;

  while((opt = getopt_long(argc, argv, "", long_options, NULL)) != -1){

    int option_index = 0;

    switch(opt){
      case 0:
        //if (long_options[option_index].flag != 0)
        break;
      case 'a':
        alpha = atof(optarg); break;
      case 'b':
        projection_length = atof(optarg); break;
      case 'c':
        triangulation_density = atof(optarg); break;
      case 'd':
        water_probe_radius = atof(optarg); break;
      case 'f':
        nano_conf_file = calloc(strlen(optarg)+1,sizeof(char));
        strcpy(nano_conf_file,optarg); break;
      case 'g':
        write_electric_field_flag = 1;
        if(optarg != NULL) strcpy(electric_field_write_file,optarg);
        break;
      case 'i':
        write_apparent_surface_charge_flag = 1;
        if(optarg != NULL) strcpy(apparent_surface_charge_write_file, optarg);
        break;
      case 'j':
        residue_list_flag = 1;
        strcpy(residue_list_file,optarg); break;
      case 'h':
      case '?':
        display_help(); exit(0);
      case ':':
        switch(optopt){
          //case 'g':
          //  strcpy(optarg,"electric_field_result_file"); break;
          //case 'i':
          //  strcpy(optarg,"apparent_surface_charge_result_file"); break;
          default:
            fprintf(stderr, "option -%c is missing a required argument\n", optopt); exit(0);
        }
      default:
        fprintf(stderr, "there was a mistake\n");
        break;
    }
  }

  char pqr_file[256];
  double dielectric_external;
  double dielectric_internal;

  if (argv[optind] == NULL || argv[optind + 1] == NULL || argv[optind + 2] == NULL) {
    fprintf(stderr, "Mandatory argument(s) missing\n"); display_help(); return -1;
  }
  else{
    strcpy(pqr_file, argv[optind]);
    dielectric_external = atof(argv[optind + 1]);
    dielectric_internal = atof(argv[optind + 2]);
  }

  physical_structure *phys_struct = calloc(1,sizeof(physical_structure));
  triangulation_structure *tri_struct = calloc(1,sizeof(triangulation_structure));
  model_parameters *md_param = calloc(1, sizeof(model_parameters));

  md_param->configuration_file_name = nano_conf_file;
  md_param->residue_list_file = residue_list_file;
  md_param->alpha = alpha;
  md_param->epsin = dielectric_internal;
  md_param->epsout = dielectric_external;
  md_param->projection_length = projection_length;
  md_param->water_probe_radius = water_probe_radius;
  md_param->triangulation_density = triangulation_density;

  if(msms_flag) prepare_structures(pqr_file, phys_struct, MSMS, tri_struct, md_param);//Not tested
  else prepare_structures(pqr_file, phys_struct, NANOSHAPER, tri_struct, md_param);

  md_param->A = estimate_A(phys_struct->residue_list, phys_struct->num_residues);
  printf("The estimated electrostatic radius (A) of this molecule is : %f\n", md_param->A);

  if(residue_list_flag){
    calculate_electric_field_normal_residue_list(read_residue_list(md_param->residue_list_file,phys_struct),phys_struct,tri_struct,md_param);
  }
  else{
    calculate_electric_field_normal_molecule(phys_struct, tri_struct, md_param);
    if(total_surface_charge_flag){
      printf("Total Surface Charge is : %f\n", calculate_total_surface_charge(tri_struct));
    }
    if(solvation_energy_flag || (!write_electric_field_flag && !write_apparent_surface_charge_flag)){
      printf("Solvation Energy is : %f\n",compute_solvation_energy(phys_struct, tri_struct));
    }
  }

	if(write_electric_field_flag){
    printf("Writing EFIELD\n");
    int write_type = EFIELD;
    if(*electric_field_write_file == '\0'){
      strcpy(electric_field_write_file,phys_struct->mol_name);
      strcat(electric_field_write_file,"_ef");
    }
    write_gvert_gface(electric_field_write_file, tri_struct, write_type);
  }
	if(write_apparent_surface_charge_flag){
    printf("Writing ASC\n");
    int write_type = ASC;
    if(*apparent_surface_charge_write_file == '\0'){
      strcpy(apparent_surface_charge_write_file ,phys_struct->mol_name);
      strcat(apparent_surface_charge_write_file ,"_asc");
    }
    write_gvert_gface(apparent_surface_charge_write_file, tri_struct, write_type);
	}

  return 0;
}
