#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <stdbool.h>
#include "calculations.h"

/**
 * Return true if the given atom is part of a residue in the given residue list
 **/
static bool atom_in_residue_list(atom*, residue**);

/* Function which computes the electric field normal at a given vertex */
void calculate_electric_field_normal_vertex(vertex *vert, physical_structure* phys_struct, model_parameters* md_param){ 

  //Necessary to reduce computation vertices that have already been computed in residue list function path
  if(vert->electric_field_normal != 0){
    return;
  }

  double vertex_vec[3] = {vert->x, vert->y, vert->z};

  double r_norm_vec[3] = {vert->x_normal, vert->y_normal, vert->z_normal};

  double r_vec[3] = {
    vertex_vec[0] + r_norm_vec[0] * md_param->projection_length,
    vertex_vec[1] + r_norm_vec[1] * md_param->projection_length,
    vertex_vec[2] + r_norm_vec[2] * md_param->projection_length
  };

  double electric_field_vertex_total = 0;

  for(int j = 0; j < phys_struct->num_residues; j++){

    residue *current_residue = phys_struct->residue_list[j];

    for(int k = 0; k < current_residue->num_atoms; k++){

      atom *current_charge = &(current_residue->atom_list[k]);

      double current_charge_vec[3] = {
        current_charge->x,
        current_charge->y,
        current_charge->z
      };

      double di_vec[3] = {
        r_vec[0] - current_charge_vec[0],
        r_vec[1] - current_charge_vec[1],
        r_vec[2] - current_charge_vec[2]
      };

      double di = magnitude(di_vec);

      double cos_phi = dot_prod(r_norm_vec, di_vec) / di;

      double current_contribution = ( 1 / ( md_param->epsout * ( 1 + md_param->alpha * ( md_param->epsin / md_param->epsout ) ) ) ) * current_charge->charge * ( ( ( md_param->alpha * ( 1 - ( md_param->epsin / md_param->epsout ) ) ) / ( ( md_param->A + md_param->projection_length ) * ( md_param->A + md_param->projection_length ) ) ) - ( cos_phi * ( ( 1 + md_param->alpha ) / ( di * di ) ) ) );

      electric_field_vertex_total += current_contribution;
    }
  }

  //vert->electric_field_normal = ELECTROSTATIC_CONVERSION_FACTOR * electric_field_vertex_total;
  vert->electric_field_normal = -1 * ELECTROSTATIC_CONVERSION_FACTOR * electric_field_vertex_total;

  //vert->surface_charge_density = ( vert->electric_field_normal * ( 1 / ( 4 * M_PI * ELECTROSTATIC_CONVERSION_FACTOR ) ) ) * ( ( md_param->epsout / md_param->epsin ) - 1 );
  vert->surface_charge_density = -1 * ( vert->electric_field_normal * ( 1 / ( 4 * M_PI * ELECTROSTATIC_CONVERSION_FACTOR ) ) ) * ( ( md_param->epsout / md_param->epsin ) - 1 );
}

void calculate_electric_field_normal_triangle(triangle* current_triangle){
  current_triangle->electric_field_normal = ( current_triangle->comprising_vertices[0]->electric_field_normal + current_triangle->comprising_vertices[1]->electric_field_normal + current_triangle->comprising_vertices[2]->electric_field_normal ) / 3;

  current_triangle->surface_charge_density = ( current_triangle->comprising_vertices[0]->surface_charge_density + current_triangle->comprising_vertices[1]->surface_charge_density + current_triangle->comprising_vertices[2]->surface_charge_density ) / 3;
}

void calculate_electric_field_normal_molecule(physical_structure* phys_struct, triangulation_structure* tri_struct, model_parameters* md_param){

  for(int i = 0; i < tri_struct->num_vertex; i++){
    calculate_electric_field_normal_vertex(&(tri_struct->vertex_list[i]), phys_struct, md_param);
  }

  for(int i = 0; i < tri_struct->num_triangles; i++){
    calculate_electric_field_normal_triangle(&(tri_struct->triangle_list[i]));
  }
}

/**
 * Return true if the given atom is part of a residue in the given residue list
 **/
static bool atom_in_residue_list(atom *atom, residue **residue_list){

  int i = 0;
  residue *cures = residue_list[i];
  while(cures != NULL){
    if(atom->containing_residue == cures){
      return true;
    }
    cures = residue_list[++i];
  }
  return false;
}

void calculate_electric_field_normal_residue_list(residue** calculate_residue_list, physical_structure* phys_struct, triangulation_structure* tri_struct, model_parameters* md_param){
  for(int i = 0; i < tri_struct->num_triangles; i++){
    if(atom_in_residue_list(phys_struct->atom_list[tri_struct->triangle_list[i].nearest_atom_index],calculate_residue_list)){
      calculate_electric_field_normal_vertex(tri_struct->triangle_list[i].comprising_vertices[0], phys_struct, md_param);
      calculate_electric_field_normal_vertex(tri_struct->triangle_list[i].comprising_vertices[1], phys_struct, md_param);
      calculate_electric_field_normal_vertex(tri_struct->triangle_list[i].comprising_vertices[2], phys_struct, md_param);
      calculate_electric_field_normal_triangle(&(tri_struct->triangle_list[i]));
    }
  }
}

void calculate_single_triangle_area(triangle* current_triangle){

    vertex *first = current_triangle->comprising_vertices[0];
    vertex *second = current_triangle->comprising_vertices[1];
    vertex *third = current_triangle->comprising_vertices[2];

    double vals[3] = {
      dist(first->x, first->y, first->z, second->x, second->y, second->z),
      dist(first->x, first->y, first->z, third->x, third->y, third->z),
      dist(third->x, third->y, third->z, second->x, second->y, second->z)
    };

    double a, b, c;

    qsort(vals, 3, sizeof(double), cmpfunc);

    c = vals[0]; b = vals[1]; a = vals[2];

    long double semi = (a + b + c)/2.;

    long double inner = semi * (semi - a) * (semi - b) * (semi - c);

    current_triangle->area = sqrt(inner);
}

void calculate_all_triangles_area(triangulation_structure* tri_struct){
  for(int i = 0; i < tri_struct->num_triangles; i++){
    calculate_single_triangle_area(&(tri_struct->triangle_list[i]));
  }
}

/* computes the total surface charge of the structure */
double calculate_total_surface_charge(triangulation_structure* tri_struct){
  double total_surface_charge = 0;
  calculate_all_triangles_area(tri_struct);
  for(int i = 0; i < tri_struct->num_triangles; i++){
    total_surface_charge += (&(tri_struct->triangle_list[i]))->area * (&(tri_struct->triangle_list[i]))->surface_charge_density;
  }
  return total_surface_charge;
}

/* computes total solvation energy of the molecules */
double compute_solvation_energy(physical_structure* phys_struct, triangulation_structure* tri_struct){

  double total_solvation_energy = 0;
  double prefactor = ELECTROSTATIC_CONVERSION_FACTOR / 2;

  if(tri_struct->triangle_list[0].area == 0) calculate_all_triangles_area(tri_struct);

  for(int j = 0; j < phys_struct->num_residues; j++){

    residue *current_residue = phys_struct->residue_list[j];

    for(int k = 0; k < current_residue->num_atoms; k++){

      atom *current_atom = &(current_residue->atom_list[k]);

      for(int t = 0; t < tri_struct->num_triangles; t++){

        triangle *current_triangle = &(tri_struct->triangle_list[t]);

        double center[3] = {
          (current_triangle->comprising_vertices[0]->x + current_triangle->comprising_vertices[1]->x + current_triangle->comprising_vertices[2]->x)/3.,
          (current_triangle->comprising_vertices[0]->y + current_triangle->comprising_vertices[1]->y + current_triangle->comprising_vertices[2]->y)/3.,
          (current_triangle->comprising_vertices[0]->z + current_triangle->comprising_vertices[1]->z + current_triangle->comprising_vertices[2]->z)/3.
        };

        long double radius = dist(current_atom->x, current_atom->y, current_atom->z, center[0], center[1], center[2]);

        total_solvation_energy += ( ( current_atom->charge * current_triangle->surface_charge_density * current_triangle->area) / fabs(radius) );
      }
    }
  }

  return prefactor * total_solvation_energy;
}
