
//////////////////////////////////////////////////////////////////////////////
// 
// Author: Grigori Sigalov <sigalov@vt.edu>, <greg_sigalov@yahoo.com>
// Modified by Dan Folescu, 2021
//
// Algorithm Details are in: 
//
// "Analytical Linearized Poisson--Boltzmann Approach 
//   for Molecular Dynamics Applications", by Sigalov, Fenley and Onufriev
// submitted to  J. Chem. Phys., 2006
//
////////////////////////////////////////////////////////////////////////////
 
// Brief algorithm: First, the moments of inertia of the molecule are
// calculated. The electrostatic size is found from the third invariant (determinant)
// of the tensor of inertia. The _principal_ moments of inertia are found
// by solving the cubic equation det(I - lambda E) = 0. The semiaxes of the
// ellipsoid that has same principal moments of inertia and mass as the
// original molecule are found.
//
// Gives a reasonable estimate of the electrostatic size. In
// most cases, you will want nothing more. Moreover, A_det has simple
// analytical derivatives, so if you are going to use derivatives of the
// electrostatic size, you should choose A_det. This option involves
// calculation of the moments of inertia of the molecule, which is a very
// straighforward procedure.  

#include <stdio.h>
#include <math.h>   
#include "structures.h"

#define DEFAULTATOMSIZE 1.5
#define ACCURACY 1.E-09 // convergence of Simpson's method, stopping condition  

#define TOL 1.E-12		// values < TOL are treated as zeros

int debug = 0;	// this setting (default) suppresses all warnings; call with key -deb if unsure about your input file

double estimate_A (residue **residues, int num_residues)
{
  double  x, y, z, r2, r3; /* r^3 (an estimate of atomic mass) */
	long long i, j;
	double xav = 0., yav = 0., zav = 0., // center-of-mass coordinates
		molecule_mass = 0., atom_mass;

    for (i = 0; i < num_residues; i++)
    {
      for (j = 0; j<residues[i]->num_atoms; j++)
      {
        r3 = residues[i]->atom_list[j].radius;
        r3 *= r3*r3;

		  xav += residues[i]->atom_list[j].x * r3; 	// atom's "mass" is simply r^3
		  yav += residues[i]->atom_list[j].y * r3; 
		  zav += residues[i]->atom_list[j].z * r3;
		  molecule_mass += r3;
	    }
    }
	
	xav /= molecule_mass;	// now it's the center
	yav /= molecule_mass;
	zav /= molecule_mass;
	
	double x2, y2, z2, atoms_MI;
	double I11 = 0., I12 = 0., I13 = 0., I22 = 0., I23 = 0., I33 = 0.;

  for (i = 0; i < num_residues; i++)
  {
	  for (j=0; j<residues[i]->num_atoms; j++)	// calculating the moments of inertia
	  {
        r2 = residues[i]->atom_list[j].radius;
        r2 *= r2;
        r3 = r2 * residues[i]->atom_list[j].radius;

		  atom_mass = r3;

		  atoms_MI = r2 / 5.;	// half atom's moment of ineria about diameter (will be added twice!)

      x = residues[i]->atom_list[j].x - xav;
      y = residues[i]->atom_list[j].y - yav;
      z = residues[i]->atom_list[j].z - zav;
		  x2 = x * x + atoms_MI;
		  y2 = y * y + atoms_MI;
		  z2 = z * z + atoms_MI;
		   
		  I11 += atom_mass * (y2 + z2);
		  I22 += atom_mass * (z2 + x2);
		  I33 += atom_mass * (x2 + y2);
		   
		  I12 -= atom_mass * x * y;		// atoms' moments do not contribute 
		  I13 -= atom_mass * x * z;		// because of symmetry
		  I23 -= atom_mass * y * z;
	  }
  }
	
	if (debug)
	{
		fprintf(stderr, "Original tensor of inertia: %12.4g %12.4g %12.4g\n", I11, I12, I13);
		fprintf(stderr, "                            %12.4g %12.4g %12.4g\n", I12, I22, I23);
		fprintf(stderr, "                            %12.4g %12.4g %12.4g\n", I13, I23, I33);
	}
	
	double det_I = I11 * I22 * I33 + 2. * I12 * I23 * I13 - I11 * I23 * I23  - I22 * I13 * I13 - I33 * I12 * I12;

	if (det_I <= 0.)
	{
	 	fprintf(stderr, "Problem with the determinant of the tensor of intertia: Det I = %g\n", det_I);
	}
	return sqrt( 2.5 / molecule_mass ) * pow( det_I, 1./6. );
}
