/***************************************************************************
* GEM -- electrostatics calculations and visualization                     *
* Copyright (C) 2006 John C. Gordon, 2021 Dan Folescu                      *
*                                                                          *
* This program is free software; you can redistribute it and/or modify     *
* it under the terms of the GNU General Public License as published by     *
* the Free Software Foundation; either version 2 of the License, or        *
* (at your option) any later version.                                      *
*                                                                          *
* This program is distributed in the hope that it will be useful,          *
* but WITHOUT ANY WARRANTY; without even the implied warranty of           *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                             *
*                                                                          *
* You should have received a copy of the GNU General Public License along  *
* with this program; if not, write to the Free Software Foundation, Inc.,  *
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.              *
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "structures.h"
#include "calculations.h"
#include <math.h>

#define PQR_LINE_SIZE 1024

void gather_mol_info(FILE *fp, physical_structure *phys_struct)
{
  int     res_num = 0;
  int     num_atoms = 0;
  int     molecule_num_atoms = 0;
  int     last_res_num = 1;
  char    *fgetsret;
  char    line[PQR_LINE_SIZE];

  //read in the first line
  fgetsret = fgets(line, PQR_LINE_SIZE, fp); int linecount = 1;

  //while there is a next line
  while(fgetsret != NULL){
    //only count atom/hetatm lines for residues
    if(strstr(line, "END") != NULL) break;
    if((strstr(line, "ATOM") == NULL) && (strstr(line, "HETATM") == NULL) && (strstr(line, "TER") == NULL)){
      fgetsret = fgets(line, PQR_LINE_SIZE, fp); linecount++; continue;
    }
    if(strstr(line, "TER") != NULL){
      (phys_struct->num_residues)++; break;
    }
    /* all we care about is the residues number */
    sscanf(
        line,
        "%*s %*d %*s %*s %d %*f %*f %*f %*f %*f %*s",
        &res_num
    );
    //if we reach a new residue, count the previous one
    if (res_num != last_res_num){
      last_res_num = res_num;
      (phys_struct->num_residues)++;
    }
    //read the next line
    fgetsret = fgets(line, PQR_LINE_SIZE, fp); linecount++;
  }
  //report the number of lines in the file
  printf("This file is %i lines long.\n", linecount);
  //rewind the file
  fseek(fp, 0, SEEK_SET);

  phys_struct->residue_list = calloc(phys_struct->num_residues, sizeof(residue*));

  last_res_num = res_num = 1;
  
  //read in the first line
  fgetsret = fgets(line, PQR_LINE_SIZE, fp);

  //while there is a next line
  while(fgetsret != NULL){
    //If we are at the end of the molecule/chain
    if((strstr(line, "END") != NULL)) break;
    if(strstr(line, "TER") != NULL){
      //update the final residue and break
      phys_struct->residue_list[last_res_num-1] = calloc(1,sizeof(residue));
      phys_struct->residue_list[last_res_num-1]->res_num = last_res_num;
      phys_struct->residue_list[last_res_num-1]->num_atoms = num_atoms;
      phys_struct->residue_list[last_res_num-1]->atom_list = calloc(num_atoms, sizeof(atom));
      phys_struct->num_atoms += num_atoms; //increment the total atoms
      break;
    }
    
    //If this line is not an END/TER, but doesn't feature any atoms, just get the next line and continue
    if((strstr(line, "ATOM") == NULL) && (strstr(line, "HETATM") == NULL)){
      fgetsret = fgets(line, PQR_LINE_SIZE, fp); continue;
    }
    //Otherwise, the line features a valid atom in the residue
    // all we care about is the residues number
    sscanf(
      line,
      "%*s %*d %*s %*s %d %*f %*f %*f %*f %*f %*s",
      &res_num
    );
    //update the residue in the list
    if (res_num != last_res_num){
      phys_struct->residue_list[last_res_num-1] = calloc(1,sizeof(residue));
      phys_struct->residue_list[last_res_num-1]->res_num = last_res_num;
      phys_struct->residue_list[last_res_num-1]->num_atoms = num_atoms;
      phys_struct->residue_list[last_res_num-1]->atom_list = calloc(num_atoms, sizeof(atom));
      phys_struct->num_atoms += num_atoms; //increment the total atoms
      num_atoms = 0; //reset num_atoms
      last_res_num++; //increment the "last residue"
    }
    //In any case, update the number of atoms in the residue (new or old), and the number of atoms in the molecule
    num_atoms++; molecule_num_atoms++;
    //read the next line
    fgetsret = fgets(line, PQR_LINE_SIZE, fp); linecount++;
  }

  //phys_struct->num_atoms = molecule_num_atoms;
  phys_struct->atom_list = calloc(molecule_num_atoms, sizeof(atom));

  printf("This molecule contains %i residues and %i atoms.\n", phys_struct->num_residues, phys_struct->num_atoms);

  /* rewind the file (again) */
  fseek(fp, 0, SEEK_SET);
}

void read_pqr(char *fname, physical_structure *phys_struct)
{
  FILE    *fp;
  char    *fgetsret;
  char    line[PQR_LINE_SIZE];

  fp = fopen(fname, "r"); //try to open pqr file

  if (!fp){
		fprintf(stderr, "There was an error when opening the PQR file");
    exit(-1);
  }

  /* gather the residue profile and allocate residue/atom lists */
  gather_mol_info(fp, phys_struct);

  /* read in the line as a string */
  fgetsret = fgets(line, PQR_LINE_SIZE, fp);

  int atom_list_count = 0;

  for (int i = 0; (fgetsret != NULL) && (i < phys_struct->num_residues); i++){

    for (int j = 0; (j < phys_struct->residue_list[i]->num_atoms); j++){

      if ((strstr(line, "ATOM") == NULL) && (strstr(line, "HETATM") == NULL)){
        continue;
      }

      //get residue and atom information
      sscanf(
        line,
        "%*s %d %*s %s %*d %f %f %f %f %f %s",
        &(phys_struct->residue_list[i]->atom_list[j].index),
        &(phys_struct->residue_list[i]->name),
        &(phys_struct->residue_list[i]->atom_list[j].x),
        &(phys_struct->residue_list[i]->atom_list[j].y),
        &(phys_struct->residue_list[i]->atom_list[j].z),
        &(phys_struct->residue_list[i]->atom_list[j].charge),
        &(phys_struct->residue_list[i]->atom_list[j].radius),
        &(phys_struct->residue_list[i]->atom_list[j].name)
      );

      //keep track of residue's geometric center in space
      phys_struct->residue_list[i]->x += phys_struct->residue_list[i]->atom_list[j].x;
      phys_struct->residue_list[i]->y += phys_struct->residue_list[i]->atom_list[j].y;
      phys_struct->residue_list[i]->z += phys_struct->residue_list[i]->atom_list[j].z;

      //atom's containing residue
      phys_struct->residue_list[i]->atom_list[j].containing_residue = phys_struct->residue_list[i];

      /* keep a running total of the charges on the residue */
      phys_struct->residue_list[i]->charge += phys_struct->residue_list[i]->atom_list[j].charge;

      /* add current atom to molecule's atom list */
      phys_struct->atom_list[atom_list_count++] = &(phys_struct->residue_list[i]->atom_list[j]);

      /* read in the line as a string */
      fgetsret = fgets(line, PQR_LINE_SIZE, fp);
    }

    //divide by number of atoms for an average geo center
    phys_struct->residue_list[i]->x /= (float) phys_struct->residue_list[i]->num_atoms;
    phys_struct->residue_list[i]->y /= (float) phys_struct->residue_list[i]->num_atoms;
    phys_struct->residue_list[i]->z /= (float) phys_struct->residue_list[i]->num_atoms;

    //add residue's charge to the total molecular charge
    phys_struct->molecular_charge += phys_struct->residue_list[i]->charge;
  }

  printf("Total molecular charge: %2.2f\n", phys_struct->molecular_charge);
  fflush(stdout);

  //populate the physical structure with molecule name
  phys_struct->mol_name = calloc(strlen(fname),sizeof(char));
  strncpy(phys_struct->mol_name,fname,strlen(fname)-4);
  phys_struct->mol_name[strlen(fname)-3] ='0'; //add null terminator
}
