/***************************************************************************
* GEM -- electrostatics calculations and visualization                     *
* Copyright (C) 2006 John C. Gordon, 2021 Dan Folescu                      *
*                                                                          *
* This program is free software; you can redistribute it and/or modify     *
* it under the terms of the GNU General Public License as published by     *
* the Free Software Foundation; either version 2 of the License, or        *
* (at your option) any later version.                                      *
*                                                                          *
* This program is distributed in the hope that it will be useful,          *
* but WITHOUT ANY WARRANTY; without even the implied warranty of           *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                             *
*                                                                          *
* You should have received a copy of the GNU General Public License along  *
* with this program; if not, write to the Free Software Foundation, Inc.,  *
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.              *
***************************************************************************/

#include <stdio.h>
#include <math.h>
#include "file_io.h"
#include "calculations.h"
#include <stdlib.h> /* for exit */
#include <string.h> /* for strcat */

#define LINE_SIZE 1024

/**********************************************************************************
 * FUNCTION: read_vertface_output -- reads in files output by an triangulation run 
 *                                                                        				*
 * INPUTS:   molname   -- the name of the molecule to run on              				*
 *                                                                        				*
 * OUTPUTS:  vertices  -- the vertices read from the .vert file           				*
 *           triangles -- the triangles read from the .face file          				*
 *           nvert     -- the number of vertices read in                  				*
 *           nface     -- the number of triangles read in                 				*
 *                                                                        				*
 *                                                                        				*
 * RETURNS: nothing                                                       				*
 **********************************************************************************/
void read_vert_face(char *molname, vertex **vertices, triangle **triangles, int *nvert, int *nface)
{
   /* local variables */
   FILE *fp; /* the file pointer to the vertex file */

   double mag;
   char line[LINE_SIZE]; /* a line in the input file to read */

   int  i,         /* counter for iterations */
        ierr,      /* the number of things read in a sscanf or scanf        */
        nv,
        nt;

   vertex  *vertx;
   triangle *tri;

   *vertices = NULL;
   *triangles = NULL;
   *nvert = *nface = 0;

   /* Open the vert file */
   strcpy(line, molname);
   strcat(line, ".vert");
   fp = fopen(line,"r");
   if (!fp) 
   {
      fprintf(stderr, "Error: could not open file %s\n", line);
      return;
   }

   /* Have a look at the header records in the vertex file.          */
   /* I suspect that the 2nd line should be parsed to determine the  */
   /* actual file contents, but the current documentation doesn't    */
   /* indicate what other items might be on the line.  So for now    */
   /* I'll assume that it must contain the number of vertices as the */
   /* first item on the line.                                        */
   fgets( line, LINE_SIZE, fp );
   fgets( line, LINE_SIZE, fp );

   /* this is the line we care about */
   fgets( line, LINE_SIZE, fp );
   sscanf( line, "%d %d", &nv, &ierr);

   /* allocate the vertex array */
   vertx = (vertex*)calloc(nv, sizeof(vertex));

   /* check for errors */
   if(vertx == NULL)
   {
      fprintf(stderr, "Error, too many vertices to process on this machine\n");
      return;
   }

   for (i=0; i< nv; i++)
   {
      fgets( line, LINE_SIZE, fp );
      ierr = sscanf
             (
                 line,
                 "%f %f %f %f %f %f %*d %d %*f", /* this is the full format of the line */
                 &vertx[i].x,
                 &vertx[i].y,
                 &vertx[i].z,
                 &vertx[i].x_normal,
                 &vertx[i].y_normal,
                 &vertx[i].z_normal,
                 &vertx[i].nearest_atom_index
             );

      /* make sure normals are unit normals */
      mag = dist(vertx[i].x_normal, vertx[i].y_normal, vertx[i].z_normal, 0., 0., 0.);

      vertx[i].x_normal /= mag;
      vertx[i].y_normal /= mag;
      vertx[i].z_normal /= mag;

      /* vert/face starts with 1 instead of 0 */
      //remember, we added 3 dummy atoms to the xyzr file
      vertx[i].nearest_atom_index -= 4;

      if (ierr != 7)
      {
         fprintf(stderr, "Error reading vertex %d", i);
         free(vertx);
         return;
      }

    }/* end for (nv) loop */

    *nvert = nv;
    *vertices = vertx;

    /* close the vertex file */
    fclose(fp);

    strcpy(line, molname);
    strcat(line, ".face");
    fp = fopen(line,"r");

    /* read header information */
    fgets(line, LINE_SIZE, fp);
    fgets(line, LINE_SIZE, fp);
    fgets(line, LINE_SIZE, fp);
    sscanf(line, "%d", &nt);

    /* allocate our triangles here */
    tri = (triangle *)calloc(nt, sizeof(triangle));

    /* now start reading triangles */
    for (i = 0; i < nt; i++){
      fgets(line, LINE_SIZE, fp);
      ierr = sscanf(line,"%d %d %d %*d %*d", &tri[i].v[0], &tri[i].v[1], &tri[i].v[2]);

      tri[i].v[0]--;
      tri[i].v[1]--;
      tri[i].v[2]--;
      //tri[i].comprising_vertices = (vertex**)malloc(3*sizeof(vertex*));
      tri[i].comprising_vertices[0] = &(vertx[tri[i].v[0]]);
      tri[i].comprising_vertices[1] = &(vertx[tri[i].v[1]]);
      tri[i].comprising_vertices[2] = &(vertx[tri[i].v[2]]);

      if (ierr != 3){
          fprintf(stderr, "Error reading from face file %s.face\n", molname);
          free(vertx);
          free(tri);
          return;
      }

      /* assign each triangle a nearest atom in a logical way */
      tri[i].nearest_atom_index = vertx[tri[i].v[0]].nearest_atom_index;
    
    }

    /* set output pointers to data retrieved */
    *nface = nt;
    *triangles = tri;

    fflush(stdout);

    /* close the face file */
    fclose(fp);

}/* end read_vertface function */
