/***************************************************************************
* GEM -- electrostatics calculations and visualization                     *
* Copyright (C) 2006 John C. Gordon, 2021 Dan Folescu                      *
*                                                                          *
* This program is free software; you can redistribute it and/or modify     *
* it under the terms of the GNU General Public License as published by     *
* the Free Software Foundation; either version 2 of the License, or        *
* (at your option) any later version.                                      *
*                                                                          *
* This program is distributed in the hope that it will be useful,          *
* but WITHOUT ANY WARRANTY; without even the implied warranty of           *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                             *
*                                                                          *
* You should have received a copy of the GNU General Public License along  *
* with this program; if not, write to the Free Software Foundation, Inc.,  *
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.              *
***************************************************************************/

#include "file_io.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

/****************************************************************************
 * FUNCTION: run_msms  -- runs the program msms                             *
 *                                                                          *
 * INPUTS: molname    -- the molecule name to be run                        *
 *         residues   -- the residues to be input to the msms algorithm     *
 *         nres       -- the number of residues being input                 *
 *         tri_per_A  -- surface resolution of the desired output           *
 *         probe_size -- the radius of the probe used to represent H20      *
 *                                                                          *
 * OUTPUTS: none                                                            *
 *                                                                          *
 * RETURNS: 1 if MSMS_EXECUTABLE_PATH is set, 0 otherwise                   *
 *                                                                          *
 ****************************************************************************/
int run_msms (physical_structure *phys_struct, model_parameters *md_param)
{
  /* local variables */
  int  size, len;
  char *mol_name = phys_struct->mol_name;
  char *fname;
  char *run;
  char *msms_exec_path;
  char *density;
  char *probe_radius;

  /* allocates memory */
  msms_exec_path = getenv("MSMS_EXECUTABLE_PATH");

  if (msms_exec_path == NULL)
  {
     return 0;
  }

  /* new memory */
  len = strlen(mol_name);
  fname = (char *)calloc(len + 6, sizeof(char));

  sprintf(fname, "%s.xyzr", mol_name);
  sprintf(density, "%2.2f", md_param->triangulation_density);
  sprintf(probe_radius, "%2.2f", md_param->water_probe_radius);

  write_xyzr(fname, phys_struct);

  /* new memory */
  size = strlen(msms_exec_path) + 56 + 2*len + strlen(density) + strlen(probe_radius);

  run = (char *)calloc(size, sizeof(char));

  sprintf
    (
       run,
       "%s -if %s -of %s -de %s -probe_radius %s -no_area >/dev/null",
       msms_exec_path,
       fname,
       mol_name,
       density,
       probe_radius
    );
  printf("running MSMS with command: %s\n", run);
  fflush(stdout);

  system(run);

  /* free up new memory */
  free(fname);
  free(run);

  return 1;
}
