/***************************************************************************
* GEM -- electrostatics calculations and visualization                     *
* Copyright (C) 2006 John C. Gordon, 2021 Dan Folescu                      *
*                                                                          *
* This program is free software; you can redistribute it and/or modify     *
* it under the terms of the GNU General Public License as published by     *
* the Free Software Foundation; either version 2 of the License, or        *
* (at your option) any later version.                                      *
*                                                                          *
* This program is distributed in the hope that it will be useful,          *
* but WITHOUT ANY WARRANTY; without even the implied warranty of           *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
* GNU General Public License for more details.                             *
*                                                                          *
* You should have received a copy of the GNU General Public License along  *
* with this program; if not, write to the Free Software Foundation, Inc.,  *
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.              *
***************************************************************************/

#include <stdio.h>
#include <math.h>
#include "file_io.h"
#include "calculations.h"
#include <stdlib.h> /* for exit */
#include <string.h> /* for strcat */

/**************************************************************************
 * FUNCTION: write_gverts  -- writes out vertices for reuse later         *
 *                                                                        *
 * INPUTS:   molname   -- the name of the molecule to run on              *
 *                                                                        *
 * OUTPUTS:  vertices  -- the vertices to write to the .gvert file        *
 *           triangles -- the triangles to write to the .gface file       *
 *           nvert     -- the number of vertices to write                 *
 *           nface     -- the number of triangles to write                *
 *                                                                        *
 * RETURNS: nothing                                                       *
 **************************************************************************/
void write_gvert_gface(char* out_file, triangulation_structure *tri_struct, Write_Type write_type)
{
  FILE *fp;
  char line[1024];
  int ierr;

  /* Open the vert file */
  strcpy(line, out_file);
  strcat(line, ".gvert");
  fp = fopen(line,"w");
  if (!fp){
      fprintf(stderr, "Error: could not open file %s for writing.\n", line);
      return;
  }
   
  if(write_type == EFIELD) fprintf (fp, "GEM calculated and writes EFIELD\n");
  
  else if(write_type == ASC) fprintf (fp, "GEM calculated and writes ASC\n");

  else{
    fprintf (stderr, "Write-type invalid.\n");
    exit(-1);
  }

  /* dump a fake NanoShaper-like header, only indicating num vertices, though */
  fprintf (fp, "#vertex\n");
  fprintf (fp, "%d\n", tri_struct->num_vertex);

  for (int i = 0; i < tri_struct->num_vertex; i++){
      if(write_type == EFIELD){
      ierr = fprintf
        (
            fp,
            "%f %f %f %f %f %f %d %d %f %lf\n", /* this is the full format of the line */
            tri_struct->vertex_list[i].x,
            tri_struct->vertex_list[i].y,
            tri_struct->vertex_list[i].z,
            tri_struct->vertex_list[i].x_normal,
            tri_struct->vertex_list[i].y_normal,
            tri_struct->vertex_list[i].z_normal,
            0,
            tri_struct->vertex_list[i].nearest_atom_index+1,
            0.,
            tri_struct->vertex_list[i].electric_field_normal
        );
      }
      if(write_type == ASC){
      ierr = fprintf
         (
            fp,
            "%f %f %f %f %f %f %d %d %f %lf\n", /* this is the full format of the line */
            tri_struct->vertex_list[i].x,
            tri_struct->vertex_list[i].y,
            tri_struct->vertex_list[i].z,
            tri_struct->vertex_list[i].x_normal,
            tri_struct->vertex_list[i].y_normal,
            tri_struct->vertex_list[i].z_normal,
            0,
            tri_struct->vertex_list[i].nearest_atom_index+1,
            0.,
            tri_struct->vertex_list[i].surface_charge_density
         );
         
      }
      if (ierr < 0){
        fprintf(stderr, "Error writing gvert file.");
        exit(-2);
      }
    }

    /* close the vertex file */
    fclose(fp);

    strcpy(line, out_file);
    strcat(line, ".gface");
    fp = fopen(line,"w");

    if (!fp){
      fprintf(stderr, "Error: could not open file %s for writing.\n", line);
      exit(-3);
    }

    /* write header information */
    fprintf (fp, "GEM saved face file\n");
    fprintf (fp, "#faces\n");
    fprintf (fp, "%d\n", tri_struct->num_triangles);

    /* now start reading triangles */
    for (int i = 0; i < tri_struct->num_triangles; i++){
      fgets(line, 1024, fp);

      ierr = fprintf
                (
                  fp,
                  "%d %d %d %d %d\n",
                  tri_struct->triangle_list[i].v[0]+1,
                  tri_struct->triangle_list[i].v[1]+1,
                  tri_struct->triangle_list[i].v[2]+1,
                  0,
                  0
                );

      if (ierr < 0){
          fprintf(stderr, "Error writing gface file.\n");
          exit(-4);
      }
    }

    fclose(fp);
}
