// Dan Folescu 2021
#ifndef __STRUCTURES_H__
#define __STRUCTURES_H__

typedef struct atom
{
  char    name[5];                /* the name of this atom */
  void    *containing_residue;    /* atom's containing residues */
  float   x;                      /* x coordinate */
  float   y;                      /* y coordinate */
  float   z;                      /* z coordinate */
  float   charge;                 /* charge of the atom */
  float   radius;                 /* radius of the atom */
  int     index;                  /* atom index from the input file */
  double  surface_area;           /* atom's portion of molecular surface | Angstrom^2 */
  int     num_triangles;          /* number of triangles on atom's portion of molecular surface */
  double  electric_field_normal;  /* atom's electric field normal value average on atom's portion of the molecular surface */
} atom;

typedef struct residue
{
  char name[5];       /* name of the residue */
  int res_num;        /* this residue number */
  int num_atoms;      /* number of atoms in the residue */
  float x, y, z;      /* atom averaged geometric center coordinates of residue */
  float charge;       /* total atomic charge of the residue */
  float rad;          /* electrostatic radius of residue(?) */
  atom *atom_list;    /* pointer to first atom in the main atom list. Only technically valid for the number of atoms in the residue */
} residue;

typedef struct vertex
{
  float x;                            /* x coordinate */
  float y;                            /* y coordinate */
  float z;                            /* z coordinate */
  float x_normal;                        /* x normal */
  float y_normal;                        /* y normal */
  float z_normal;                        /* z normal */
  int nearest_atom_index;             /* the atom nearest to this vertex */
  double electric_field_normal;       /* electric field at the vertex */
  double surface_charge_density;      /* surface charge at the vertex */
} vertex;

typedef struct triangle
{
  int v[3];                       /* the three vertices making up a triangle */
  vertex *comprising_vertices[3];   /* List of points to comprising vertices */
  double electric_field_normal;   /* triangle electric field normal values */
  double surface_charge_density;  /* triangle surface charge density */
  double area;                    /* triangle surface area */
  int nearest_atom_index;         /* nearest atom by consensus(v[0].nearest_atom) */
} triangle;

typedef struct physical_structure
{
  char* mol_name;                  /* name of the molecule in question */
  int num_residues;                /* number of residues in the molecule */
  residue **residue_list;            /* residues making up the molecule */
  int num_atoms;                   /* number of atoms in the molecule */
  atom **atom_list;                  /* atoms making up the molecule */
  double molecular_charge;
} physical_structure;

typedef struct triangulation_structure
{
  char* mol_name;                  /* name of the molecule in question */
  int num_triangles;
  triangle *triangle_list;
  int num_vertex;
  vertex *vertex_list;
} triangulation_structure;

typedef struct model_parameters
{
  char   *configuration_file_name;  /* file name for nanoshaper config. if not set, defaults to NULL */
  char   *residue_list_file;        /* file name for residue list. if not set, defaults to NULL */
  double alpha;                     /* alpha constant */
  double epsin;                     /* inner dielectric constant */
  double epsout;                    /* outer dielectric constant */
  double A;                         /* electrostatic radius/"molecular size" */
  double projection_length;         /* electrostatic radius/"molecular size" */
  double water_probe_radius;        /* rolling water probe radius */
  double triangulation_density;     /* triangulation density - means different things for MSMS/NanoShaper triangulation */
} model_parameters;

#endif
